function [Simu] = Tut1_Ex3(rho,n_0,N,graph)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Tutorial 1 : Exercise Three
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Inputs : 
%%%%% rho : correlation between the two parameters.
%%%%% n_0 : size of the burn-in phase
%%%%% N : size of the MCMC iterations after the burn-in phase
%%%%% graph : if =1, several convergence graphics are displayed at the end of
%%%%% the simulation (by default = 0)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Outputs :
%%%%% Simu is a structure containing :
%%%%% post_theta : MCMC Posterior draws of theta's.
%%%%% post_mean : is the posterior mean of theta's
%%%%% var_theta : denotes the posterior variance of theta's
%%%%% poststd_theta : denotes the posterior standard deviations of theta's
%%%%% post_rho : stands for the posterior correlation of theta's
%%%%% test_Geweke : Geweke's test at 95%. For each posterior sample, it
%%%%% checks if the difference of posterior means of two subsamples are
%%%%% statistically equal (=0) or different (=1).
%%%%% Direct_sampling : contains independent samples of theta's
%%%%% cusum is a structure that contains relevant information for the cusum
%%%%% plots
%%%%% var_mean_theta : is the variance of the posterior mean of theta's.
%%%%% The computation is done by batch means.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%% By default, set the variable graph = 0.
if(nargin<4)
    graph = 0;
end

disp('**********************************');
disp('******** Direct Sampling  ********');
disp('**********************************');
Sigma = [1 rho;rho 1];
Direct_sampling = mvnrnd([0 0],Sigma,N);
Simu.Direct_sampling = Direct_sampling;

%%%%%%%%%%%
%%%% MCMC Initialisation
%%%%%%%%%%%
nb_MCMC = n_0+N;
burn_in = n_0;
post_theta = zeros(N,2);
iter = 1;
theta = [0 0]';

disp('**********************************');
disp('***** BEGINNING OF THE MCMC ******');
disp('**********************************');
for i=1:nb_MCMC
    progressbar(i/nb_MCMC);
    if(i>burn_in)
        post_theta(iter,:) = theta';
        iter = iter +1;
    end
    theta(1) = rho*theta(2) + sqrt((1-rho^2))*randn(); %%% Update theta_1|theta_2.
    theta(2) = rho*theta(1) + sqrt((1-rho^2))*randn(); %%% Update theta_2|theta_1.
end
%%%%%%%%%%%%%
%%% Gathering all the statistics into the structure Simu
%%%%%%%%%%%%%
Simu.post_theta = post_theta;
Simu.post_mean = mean(post_theta);
Simu.var_theta = var(post_theta);
Simu.poststd_theta = sqrt(Simu.var_theta);
post_Sigma = cov(post_theta);
Simu.post_rho = post_Sigma(2,1); 
Simu.test_Geweke = Geweke_test(post_theta);
disp('Geweke test ')
disp(Simu.test_Geweke);

%%%%%%%%%%%%%
%%% Testing for the convergence by cusum plot and ACF
%%%%%%%%%%%%%
[Simu.cusum] = cusum_plot(post_theta,graph);
[acf] = autocorrelation_time(Simu.post_theta);
Simu.var_mean_theta = acf'.*Simu.var_theta/N;
if(graph==1)
   figure;
   plot(Direct_sampling(:,1),Direct_sampling(:,2),'.r'),hold on,plot(Simu.post_theta(:,1),Simu.post_theta(:,2),'.'),legend('Direct sampling','MCMC draws');
   figure;
   subplot(2,1,1),autocorr(post_theta(:,1),200),title('Sample ACF of \theta_1');
   subplot(2,1,2),autocorr(post_theta(:,2),200),title('Sample ACF of \theta_2');
   figure;
   p = gkde2(Simu.post_theta);
   surf(p.x,p.y,p.pdf),title('MCMC Posterior density of \theta');
   figure;
   p = gkde2(Direct_sampling);
   surf(p.x,p.y,p.pdf),title('Direct sampling Posterior density of \theta');
end
